#include <stdio.h>
#include <string.h>

#include "output.h"
#include "xmalloc.h"

int output_ctrl_init(output_ctrl_t *oc,
		      char *dirname, char *filename)
{
  /* begin with 2 output_t:s  */
  oc->outputs=xmalloc(sizeof(output_t)*2);

  oc->dirname=xmalloc(strlen(dirname)+1);
  strcpy(oc->dirname, dirname);

  oc->basename=xmalloc(strlen(filename)+1);
  strcpy(oc->basename, filename);

  /* strip .ogg extension if it's there */
  if(strcasecmp((filename+strlen(filename)-4), ".ogg")==0){
    oc->basename[strlen(filename)-4]='\0';
  }

  oc->idcount=0;
  oc->outputs_size=2;
  oc->outputs_used=0;

  return 1;
}

int output_ctrl_free(output_ctrl_t *oc)
{
  int i;
  for(i=0; i<oc->outputs_used; i++)
    fclose(oc->outputs[i].file);

  free(oc->outputs);
  free(oc->dirname);
  free(oc->basename);

  return 1;
}

output_t *output_ctrl_output_new(output_ctrl_t *oc, int chain_c, int group_c)
{
  output_t *op;
  int fnlen, fnret;

  if(oc->outputs_used==oc->outputs_size){
    /* allocate room for 2 more output_t:s */
    oc->outputs=xrealloc(oc->outputs,
			  sizeof(output_t)*(oc->outputs_size+2));
    oc->outputs_size+=2;
  }

  op=&oc->outputs[oc->outputs_used++];

  fnlen=strlen(oc->dirname)+strlen(oc->basename)+16;
  op->filename=xmalloc(fnlen);

  while(1){
    if(chain_c)
      if(group_c)
	fnret=snprintf(op->filename, fnlen, "%s/%s.c%02x.g%02x.ogg",
		       oc->dirname, oc->basename, chain_c, group_c);
      else
	fnret=snprintf(op->filename, fnlen, "%s/%s.c%02x.ogg",
		       oc->dirname, oc->basename, chain_c);
    else
      if(group_c)
	fnret=snprintf(op->filename, fnlen, "%s/%s.g%02x.ogg",
		       oc->dirname, oc->basename, group_c);
      else
	fnret=snprintf(op->filename, fnlen, "%s/%s.junk",
		       oc->dirname, oc->basename);

    if(fnret>=fnlen){
      fnlen=fnret+1;
      op->filename=xrealloc(op->filename, fnlen);
      /* try again */
      continue;
    }
    break;
  }

  op->file=fopen(op->filename, "w+");
  if(op->file==NULL){
    fprintf(stderr, "Failed to open %s for writing.\n", op->filename);
    exit(1);
  }

  op->id=oc->idcount++;
  op->count=1;

  return op;
}

int output_ctrl_output_free(output_ctrl_t *oc, int id)
{
  int i;
  for(i=0; i<oc->outputs_used; i++){
    if(oc->outputs[i].id==id){
      oc->outputs[i].count--;
      if(oc->outputs[i].count==0){
	fclose(oc->outputs[i].file);
	free(oc->outputs[i].filename);

	oc->outputs_used--;

	/* if this isn't the last, shift back other output_t:s */
	if(oc->outputs_used > i)
	  memmove(&oc->outputs[i],
		  &oc->outputs[i+1],
		  sizeof(output_t)*(oc->outputs_used-i));
      }
      return 1;
    }
  }
  return 0;
}

int output_page_write(output_t *op, ogg_page *og)
{
  if(fwrite(og->header, og->header_len, 1, op->file)==0)return 0;
  if(fwrite(og->body, og->body_len, 1, op->file)==0)return 0;

  return 1;
}
